/* Arduino firmware for the automatic MUFF v2.0 microscope positioner. */
/* Created by Gustavo Lelis (UFF) and Jorge Stolfi (UNICAMP), 2017-2018. */
/* Last edited on 2018-08-28 20:00:30 by stolfilocal */

#include <AccelStepper.h>
#include <muff_utils.h>
#include <muff_comandos.h>

// Estado interno do firmware:

int estados_dos_leds[num_bytes_leds]; // Bytes cujos bits indicam os estados correntes dos LEDs.

// Parametros do motor de passo principal (a ajustar experimentalmente):

int motor1_maxVel = 200;    // Velocidade maxima (passos/segundo).
  
int motor1_maxAcel = 500;   // Aceleracao maxima (passos/segundo^2).

int desloc_ajuste = 10000;  // Passos a deslocar nos comandos '1' e '2'. 

int desloc_padrao = 0;      // Passos a deslocar no comando '5', definido pelo comando '4'.

AccelStepper motor1 = NULL; // Configuracao e estado do motor.

void setup()
  { inicializa_porta_serial();
  
    inicializa_leds(estados_dos_leds);
    
    motor1 = inicializa_motor1(motor1_maxVel, motor1_maxAcel);

    // Prompt em caso de interacao direta com usuario
    Serial.println("# Digite comando ('1', '2', etc) e clique em ENVIAR...");
  }

void processa_comando(int comando)
  {
    mostra_comando(comando);
    if (comando == '1')
      { comando_aciona_motor(motor1, +desloc_ajuste, 0); }
    else if (comando == '2')
      { comando_aciona_motor(motor1, -desloc_ajuste, 0); }
    else if (comando == '3')
      { comando_para_motor(motor1); }
    else if (comando == '4')
      { comando_define_desloc_padrao(&desloc_padrao); }
    else if (comando == '5')
      { comando_aciona_motor(motor1, desloc_padrao, 1); }
    else if (comando == '+')
      { comando_aciona_leds(1,estados_dos_leds); }
    else if (comando == '-')
      { comando_aciona_leds(0,estados_dos_leds); }
    else
      { muff_erro("comando invalido");  }

    // Notifica o usuario de que ocomando foi executado:
    Serial.print('0');
  }

void loop(void)
  // Loop principal do firmware. 
  { // Gera pulsos para o motor se e como necessario:
    if (motor1.isRunning())
      { // Motor estah em movimento:
        motor1.run();
      }
    else
      { // Motor estah parado, desligue alimentacao para poupar energia:
        motor1.disableOutputs();
      }
    // Verifica se ha comando:
    if (Serial.available() > 0) 
      { // Chegou um comando:
        int comando = Serial.read();
        processa_comando(comando);
      }
  }
      

